unit Winsaver;

interface

uses 
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;
type

TWinSaver = class(TComponent)
 private
	FIniFileName: string;
        FIsLocal: Boolean;
	FIniSect: string;
        FIsSaveSize: Boolean;
        FIsSaveLocation: Boolean;
        FIsSaveState: Boolean;
        FIsSaveVisible: Boolean;

        FOnShow : TNotifyEvent;
        FOnClose : TCloseEvent;
        FMyOwner : TObject;

        procedure OpenIniFile;
        procedure SaveWindow(TheForm: TForm);
        procedure RestoreWindow(TheForm: TForm);
 public
	constructor Create(AOwner: TComponent); override;
        Destructor Destroy; override;
        procedure Save;
        procedure Restore;
        procedure MySave(Sender : Tobject; var Action : TCloseAction);
        procedure MyRestore(Sender : Tobject);
 published
        property IniFileName: string read FIniFileName write FIniFileName;
        property LocalDir: Boolean read FIsLocal write FIsLocal default False;
        property IniSection: string read FIniSect write FIniSect;
        property SaveSize: Boolean read FIsSaveSize write FIsSaveSize default True;
        property SaveLocation: Boolean read FIsSaveLocation write FIsSaveLocation default True;
        property SaveState: Boolean read FIsSaveState write FIsSaveState default True;
        property SaveVisible: Boolean read FIsSaveVisible write FIsSaveVisible default True;
end;

procedure register;

implementation

uses IniFiles;

var
   Ini: TIniFile;

constructor TWinSaver.Create(AOwner: TComponent);
begin

	inherited Create(AOwner);

        { initial property value }

        FIsLocal := False;
        FIsSaveSize := True;
        FIsSaveLocation := True;
        FIsSaveState := True;
        FIsSaveVisible := True;
        FIniFileName := ChangeFileExt(ExtractFileName(AOwner.Name), '.INI');
        FIniSect := 'WinSaver';

        if not (csDesigning in ComponentState) then begin
           FMyOwner := AOwner;
           FOnShow  := (AOwner as TForm).OnShow;
           FOnClose := (AOwner as TForm).OnClose;
           (AOwner as TForm).OnShow := MyRestore;
           (AOwner as TForm).OnClose := MySave;
        end;
end;

Destructor TWinSaver.Destroy;
begin
  if not (csDesigning in ComponentState) then begin
     (FMyOwner as TForm).OnShow := FOnShow;
     (FMyOwner as TForm).OnClose := FOnClose;
  end;

  inherited Destroy;
end;

procedure TWinSaver.MySave(Sender : Tobject; var Action : TCloseAction);
var i: Integer;
begin
     if Assigned(FOnClose) then
        FOnClose(Sender, Action);

     OpenIniFile;

     try

        with Application do
           for i := 0 to ComponentCount-1 do
              if (Components[i] is TForm) then
                 SaveWindow(TForm(Components[i]));

     finally
        Ini.free;
     end;
end;


procedure TWinSaver.SaveWindow(TheForm: TForm);
begin
     with TheForm do begin
          if FIsSaveSize then begin
             Ini.WriteInteger(FIniSect, ClassName + '_Width', Width);
             Ini.WriteInteger(FIniSect, ClassName + '_Height', Height);
          end;

          if FIsSaveLocation then begin
             Ini.WriteInteger(FIniSect, ClassName + '_Top', Top);
             Ini.WriteInteger(FIniSect, ClassName + '_Left', Left);
          end;

          if FIsSaveState then
             case TheForm.WindowState of
                  wsMinimized: Ini.WriteInteger(FIniSect, ClassName + '_WindowState', 1);
                  wsNormal:    Ini.WriteInteger(FIniSect, ClassName + '_WindowState', 2);
                  wsMaximized: Ini.WriteInteger(FIniSect, ClassName + '_WindowState', 3);
             end;

          if FIsSaveVisible then
            Ini.WriteBool(FIniSect, ClassName + '_Visible',Visible);

     end;
end;

procedure TWinSaver.Save;
var
   Action: TCloseAction;
begin
     MySave(FMyOwner, Action);
end;

procedure TWinSaver.Restore;
begin
     MyRestore(FMyOwner);
end;

procedure TWinSaver.MyRestore(Sender : Tobject);
var i: Integer;
begin
   if Assigned(FOnShow) then
      FOnShow(Sender);

   OpenIniFile;
   try

      with Application do
         for i := 0 to ComponentCount-1 do
            if (Components[i] is TForm) then
               RestoreWindow(TForm(Components[i]));

   finally
      Ini.free;
   end;
end;

procedure TWinSaver.RestoreWindow(TheForm: TForm);
begin
     with TheForm do begin
          if FIsSaveSize then begin
             Width := Ini.ReadInteger(FIniSect, ClassName + '_Width', Width);
             Height := Ini.ReadInteger(FIniSect, ClassName + '_Height', Height);
          end;

          if FIsSaveLocation then begin
             Top := Ini.ReadInteger(FIniSect, ClassName + '_Top', Top);
             Left := Ini.ReadInteger(FIniSect, ClassName + '_Left', Left);
          end;

          if FIsSaveState then
             case Ini.ReadInteger(FIniSect, ClassName + '_WindowState', 2) of
                  1: WindowState := wsMinimized;
                  2: WindowState := wsNormal;
                  3: WIndowState := wsMaximized;
             end;

          if FIsSaveVisible then
            Visible:=Ini.ReadBool(FIniSect, ClassName + '_Visible', Visible);

     end;
end;



procedure TWinSaver.OpenIniFile;
var
   buf: PChar;
   RealFileName: string;
begin
        if FIniFileName = '' then     { if IniFile property is empty , set to appname.ini}
           FIniFileName := ChangeFileExt(ExtractFileName(Application.ExeName), '.INI')
        else        { change to '.ini' extesion }
           FIniFileName := ChangeFileExt(ExtractFileName(FIniFileName), '.INI');

        if FIsLocal = True then
           RealFileName := ExtractFileDir(ExpandFilename(Application.Exename)) + '\' + FIniFileName
        else begin
           GetMem(buf,256);
           try
              GetWindowsDirectory(buf,256);
              RealFileName := StrPas(buf) + '\' + FIniFileName;
           finally
              FreeMem(buf,256);
           end;
        end;

        if FIniSect = '' then        {if IniSection property is empty, set to [WinSaver]}
           FIniSect := 'WinSaver';

       	Ini := TIniFile.Create(RealFileName);
end;

procedure register;
begin
     RegisterComponents('Samples',[TWinSaver]);
end;

end.
